/*
   Alg lui Lee - Clasic
   Programare Dinamica + Greedy
   O(n^4) 
   
   
   Ex. Se da o matrice a[][] nxm   cu valori nr naturale.
   *   Sa se determine suma minima a unui drum de la punctul de plecare (ip, jp), 
   *   la punctul de sosire (is, js).
       
*/
#include <iomanip>
#include <vector>
#include <fstream> // Directive de Precompilare (de Preprocesare)
#include <queue>
#include <tuple>
using namespace std;

ifstream fin("lee_sume.in");
ofstream fout("lee_sume");

// type alias-uri

using VI  = vector<int>;
using VVI = vector<VI>;     // tipul matrice de int
using VC  = vector<char>;
using VVC = vector<VC>;     // matrice de caractere
using TI  = tuple<int, int, int>;

const int Inf = 0x3f3f3f3f,
	      di[] = {-1, 0, 1, 0},
	      dj[] = {0, 1, 0, -1};
	      
// matrici vide!
VVI a;   // matricea de valori care se citesc
VVI c;   // c[i][j] = distanta minima (nr minim de pasi)
                   // de la (ip, jp) la (i, j)
int ip, jp, is, js;
int n, m;

void CitesteDate();
void Lee(int ip, int jp, VVI& c); // obligatoriu c este prin referinta!
bool Ok(int i, int j);
void  Debug();

int main()
{
	CitesteDate();
	Lee(ip, jp, c);
	
	fout << c[is][js] << '\n';
	
	Debug();
}

// directive de Precompilare
#define iv (i + di[d])
#define jv (j + dj[d])

// O(n * m)
void Lee(int ip, int jp, VVI& c)
{
	priority_queue<TI, vector<TI>, greater<TI>> Q; // min-heap
	
	c = VVI(n + 1, VI(m + 1, Inf)); // aici se creaza matricea!
	
    // plasam celula de plecare in coada
	c[ip][jp] = a[ip][jp]; 
	Q.push({c[ip][jp], ip, jp});
	
	while (!Q.empty())  // cat timp coada nu este vida
	{
		auto [cost, i, j] = Q.top();
		Q.pop();       // Stergem primul element din coada 
		
		if (cost > c[i][j]) // IMPORTANT!
			continue;
			
		for (int d = 0; d < 4; ++d)
			if (Ok(iv,jv) && c[iv][jv] > c[i][j] + a[iv][jv])
			{
				c[iv][jv] = c[i][j] + a[iv][jv];
				Q.push({c[iv][jv], iv, jv});   
			}
	}
}

bool Ok(int i, int j)
{
	if (i < 1 || i > n || j < 1 || j > m) // in afara matricei
		return false;
		
	return true;
}

void CitesteDate()
{
	fin >> n >> m;
	
	a = VVI(n + 1, VI(m + 1)); // cream matricea L[][]
	
	fin >> ip >> jp >> is >> js;
	for (int i = 1; i <= n; ++i)
		for (int j = 1; j <= m; ++j)
			fin >> a[i][j];
}

void  Debug()
{
	for (int i = 1; i <= n; ++i, fout << '\n')
	{
		for (int j = 1; j <= m; ++j)
			if (c[i][j] == Inf)
				fout << " oo";
			else
				fout << setw(3) << c[i][j];
	}
}





